@@ -23,6 +23,15 @@
 ** See the GNU General Public License for more details.
 **
 ** $Log: acctproc.c,v $
+** Revision 1.20  2005/11/04 14:17:10  gerlof
+** Improved recognition of certain version process accounting file.
+**
+** Revision 1.19  2005/10/31 12:44:58  gerlof
+** Support account-record version 3 (used by Mandriva).
+**
+** Revision 1.18  2005/10/31 08:55:05  root
+** *** empty log message ***
+**
 ** Revision 1.17  2004/12/14 15:05:00  gerlof
 ** Implementation of patch-recognition for disk and network-statistics.
 **
@@ -79,7 +88,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: acctproc.c,v 1.17 2004/12/14 15:05:00 gerlof Exp $";
+static const char rcsid[] = "$Id: acctproc.c,v 1.20 2005/11/04 14:17:10 gerlof Exp $";
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -104,10 +113,12 @@ static const char rcsid[] = "$Id: acctproc.c,v 1.17 2004/12/14 15:05:00 gerlof E
 static	char 	acctatop;	/* boolean: atop's own accounting busy ?  */
 static	off_t	acctsize;	/* previous size of account file	  */
 static	int	acctrecsz;	/* size of account record		  */
+static	int	acctversion;	/* version of account record layout	  */
 int   		acctfd   = -1;	/* fd of account file       		  */
 
 static count_t 	acctexp (comp_t  ct);
 static count_t	acctexp2(comp2_t ct);
+static int	acctvers(int);
 
 struct pacctadm {
 	char		*name;
@@ -147,7 +158,7 @@ struct sembuf	semclaim = {0, -1, SEM_UNDO},
 int
 acctswon(void)
 {
-	int			i, j, semid, dev;
+	int			i, j, semid;
 	static ushort 		vals[] = {1, SEMTOTAL};
 	union {ushort *array;}	arg = {vals};
 	struct stat		statbuf;
@@ -168,8 +179,7 @@ acctswon(void)
 		/*
 		** at least one file is present; check if it is really in use
 		** at this moment for accounting by forking a child-process
-		** which immediately finishes in order to force a new
-		** accounting-record
+		** which immediately finishes to force a new accounting-record
 		*/
 		if ( fork() == 0 )
 			exit(0);	/* let the child finish */
@@ -180,6 +190,9 @@ acctswon(void)
 		{
 			if ( stat(pacctadm[i].name, &statbuf) == 0)
 			{
+				/*
+				** accounting-file has grown?
+				*/
 				if (statbuf.st_size > pacctadm[i].stat.st_size)
 				{
 					/*
@@ -196,32 +209,15 @@ acctswon(void)
 						return 0;
 					}
 
-					/*
-					** determine the acctrec size being
-					** used (patched kernel?)
-					*/
-					dev = statbuf.st_size -
-					      pacctadm[i].stat.st_size;
-
-					if (dev % sizeof(struct acct))
+					if ( !acctvers(acctfd) )
 					{
-					   acctrecsz = sizeof(struct acct_atop);
-					   kernpatch |= PATCHACCT;
-					}
-					else
-					{
-					   acctrecsz = sizeof(struct acct);
+						fprintf(stderr,
+	                				"warning: no process "
+						        "exit detection\n");
+						sleep(3);
+						return 0;
 					}
 
-					/*
-					** accounting successfully switched on
-					*/
-					acctsize = acctprocnt() * acctrecsz;
-	
- 					/*
-					** reposition to actual file-size
-					*/
-					(void)lseek(acctfd, acctsize, SEEK_SET);
 					return 1;
 				}
 			}
@@ -348,7 +344,7 @@ acctswon(void)
 	acctatop = 1;
 
 	/*
-	** determine if patch for extended accounting is installed
+	** determine version of accounting-record
 	*/ 
 	(void) fstat(acctfd, &statbuf);
 
@@ -361,35 +357,56 @@ acctswon(void)
 			exit(0);	/* let the child finish */
 
 		(void) wait((int *) 0); /* let the parent wait  */
-
-		(void) fstat(acctfd, &statbuf);
-
-		if (statbuf.st_size % sizeof(struct acct     ) == 0  &&
-		    statbuf.st_size % sizeof(struct acct_atop) == 0    )
-		{
-			if ( fork() == 0 )
-				exit(0);	/* let the child finish */
-
-			(void) wait((int *) 0); /* let the parent wait  */
-		}
 	}
 
-	if (statbuf.st_size % sizeof(struct acct))
+	acctvers(acctfd);
+
+	return 1;
+}
+
+/*
+** determine the version of the accounting-record layout/length
+** and reposition the seek-pointer to the end of the accounting file
+*/
+static int
+acctvers(int fd)
+{
+	struct acct 	tmprec;
+
+	/*
+	** read first record from accouting file to verify
+	** the second byte (always contains version number)
+	*/
+	if ( read(fd, &tmprec, sizeof tmprec) < sizeof tmprec)
+		return 0;
+
+	switch (tmprec.ac_version)
 	{
-	 	acctrecsz = sizeof(struct acct_atop);
-		kernpatch |= PATCHACCT;
-	}
-	else
-	{
-		acctrecsz = sizeof(struct acct);
+	   case 3:
+	   	acctrecsz   = sizeof(struct acct_v3);
+		acctversion = 3;
+		break;
+
+	   case 6:
+	   	acctrecsz   = sizeof(struct acct_atop);
+		acctversion = 6;
+		kernpatch  |= PATCHACCT;  
+		break;
+
+	   default:
+	   	acctrecsz   = sizeof(struct acct);
+		acctversion = 2;
 	}
 
+	/*
+	** accounting successfully switched on
+	*/
 	acctsize = acctprocnt() * acctrecsz;
 
- 	/*
-	** reposition to actual file size
+	/*
+	** reposition to actual file-size
 	*/
-	(void) lseek(acctfd, acctsize, SEEK_SET);
+	(void)lseek(fd, acctsize, SEEK_SET);
 
 	return 1;
 }
@@ -490,8 +507,9 @@ acctphotoproc(struct pstat *accproc, int nrprocs)
 {
 	register int 		nrexit;
 	register struct pstat 	*api;
-	struct acct_atop	acctrec_atop;
 	struct acct 		acctrec;
+	struct acct_v3 		acctrec_v3;
+	struct acct_atop	acctrec_atop;
 
 	/*
 	** if accounting not supported, skip call
@@ -504,8 +522,9 @@ acctphotoproc(struct pstat *accproc, int nrprocs)
 	*/
 	for  (nrexit=0, api=accproc; nrexit < nrprocs; nrexit++, api++)
 	{
-		if ( !(kernpatch & PATCHACCT) )
+		switch (acctversion)
 		{
+		   case 2:
 			if ( read(acctfd, &acctrec, acctrecsz) < acctrecsz )
 				break;	/* unexpected end of account file */
 
@@ -513,9 +532,10 @@ acctphotoproc(struct pstat *accproc, int nrprocs)
 			** fill process info from accounting-record
 			*/
 			api->gen.state  = 'E';
+			api->gen.nthr   = 1;
 			api->gen.excode = acctrec.ac_exitcode;
-			api->gen.ruid   = acctrec.ac_uid;
-			api->gen.rgid   = acctrec.ac_gid;
+			api->gen.ruid   = acctrec.ac_uid16;
+			api->gen.rgid   = acctrec.ac_gid16;
 			api->gen.btime  = acctrec.ac_btime;
 			api->cpu.stime  = acctexp(acctrec.ac_stime);
 			api->cpu.utime  = acctexp(acctrec.ac_utime);
@@ -525,9 +545,33 @@ acctphotoproc(struct pstat *accproc, int nrprocs)
 			api->dsk.rio    = acctexp(acctrec.ac_rw);
 
 			strcpy(api->gen.name, acctrec.ac_comm);
-		}
-		else
-		{
+			break;
+
+		   case 3:
+			if ( read(acctfd, &acctrec_v3, acctrecsz) < acctrecsz )
+				break;	/* unexpected end of account file */
+
+			/*
+			** fill process info from accounting-record
+			*/
+			api->gen.state  = 'E';
+			api->gen.pid    = acctrec_v3.ac_pid;
+			api->gen.nthr   = 1;
+			api->gen.excode = acctrec_v3.ac_exitcode;
+			api->gen.ruid   = acctrec_v3.ac_uid;
+			api->gen.rgid   = acctrec_v3.ac_gid;
+			api->gen.btime  = acctrec_v3.ac_btime;
+			api->cpu.stime  = acctexp(acctrec_v3.ac_stime);
+			api->cpu.utime  = acctexp(acctrec_v3.ac_utime);
+			api->mem.minflt = acctexp(acctrec_v3.ac_minflt);
+			api->mem.majflt = acctexp(acctrec_v3.ac_majflt);
+			api->mem.iosw   = acctexp(acctrec_v3.ac_swaps);
+			api->dsk.rio    = acctexp(acctrec_v3.ac_rw);
+
+			strcpy(api->gen.name, acctrec_v3.ac_comm);
+			break;
+
+		   case 6:
 			if ( read(acctfd, &acctrec_atop, acctrecsz) <acctrecsz)
 				break;	/* unexpected end of account file */
 
@@ -536,6 +580,7 @@ acctphotoproc(struct pstat *accproc, int nrprocs)
 			*/
 			api->gen.state  = 'E';
 			api->gen.pid    = acctrec_atop.ac_pid;
+			api->gen.nthr   = 1;
 			api->gen.excode = acctrec_atop.ac_exitcode;
 			api->gen.ruid   = acctrec_atop.ac_uid;
 			api->gen.rgid   = acctrec_atop.ac_gid;
