@@ -10,13 +10,29 @@
 ** indicated interval-time a snapshot is taken of the system-level and
 ** process-level counters and the deviations are calculated and
 ** visualized for the user.
-** ================================================================
+** ==========================================================================
 ** Author:      Gerlof Langeveld - AT Computing, Nijmegen, Holland
 ** E-mail:      gerlof@ATComputing.nl
 ** Date:        November 1996
 ** Linux-port:  June 2000
-** Modifications:
-**		May 2001 - Ported to kernel 2.4
+** Modified: 	May 2001 - Ported to kernel 2.4
+** --------------------------------------------------------------------------
+** Copyright (C) 2000-2005 Gerlof Langeveld
+**
+** This program is free software; you can redistribute it and/or modify it
+** under the terms of the GNU General Public License as published by the
+** Free Software Foundation; either version 2, or (at your option) any
+** later version.
+**
+** This program is distributed in the hope that it will be useful, but
+** WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with this program; if not, write to the Free Software
+** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+** --------------------------------------------------------------------------
 **
 ** After initialization, the main-function calls the ENGINE.
 ** For every cycle (so after another interval) the ENGINE calls various 
@@ -98,17 +114,13 @@
 ** can be linked with 'atop'; the one to use can eventually be chosen
 ** at runtime. 
 **
-** This program is free software; you can redistribute it and/or modify it
-** under the terms of the GNU General Public License as published by the
-** Free Software Foundation; either version 2, or (at your option) any
-** later version.
-**
-** This program is distributed in the hope that it will be useful, but
-** WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-** See the GNU General Public License for more details.
-**
 ** $Log: atop.c,v $
+** Revision 1.22  2005/10/28 09:50:29  gerlof
+** All flags/subcommands are defined as macro's.
+**
+** Revision 1.21  2005/10/21 09:48:48  gerlof
+** Per-user accumulation of resource consumption.
+**
 ** Revision 1.20  2004/12/14 15:05:38  gerlof
 ** Implementation of patch-recognition for disk and network-statistics.
 **
@@ -173,7 +185,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: atop.c,v 1.20 2004/12/14 15:05:38 gerlof Exp $";
+static const char rcsid[] = "$Id: atop.c,v 1.22 2005/10/28 09:50:29 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -190,11 +202,13 @@ static const char rcsid[] = "$Id: atop.c,v 1.20 2004/12/14 15:05:38 gerlof Exp $
 #include <string.h>
 #include <sys/time.h>
 #include <sys/resource.h>
+#include <regex.h>
 
 #include "atop.h"
 #include "acctproc.h"
 #include "photoproc.h"
 #include "photosyst.h"
+#include "showgeneric.h"
 
 #define	allflags	"ab:cde:fghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
 #define	PROCCHUNK	100	/* portion-size for process-table expansion */
@@ -688,14 +702,17 @@ engine(void)
 void
 prusage(char *myname)
 {
-	printf("Usage: %s [-flags] [interval [samples]]\n", myname);
+	printf("Usage: %s [-flags] [interval [samples]]\n",
+					myname);
 	printf("\t\tor\n");
-	printf("Usage: %s -w  file  [-S] [-a] [interval [samples]]\n", myname);
-	printf("       %s -r [file] [-b hh:mm] [-e hh:mm] [-flags]\n", myname);
+	printf("Usage: %s -w  file  [-S] [-%c] [interval [samples]]\n",
+					myname, MALLPROC);
+	printf("       %s -r [file] [-b hh:mm] [-e hh:mm] [-flags]\n",
+					myname);
 	printf("\n");
 	printf("\tgeneric flags:\n");
-	printf("\t  -a  show or log all processes (i.s.o. active processes "
-	                "only)\n");
+	printf("\t  -%c  show or log all processes (i.s.o. active processes "
+	                "only)\n", MALLPROC);
 
 	(*vis.show_usage)();
 
@@ -714,7 +731,7 @@ prusage(char *myname)
 	printf("If the interval-value is zero, a new sample can be\n");
 	printf("forced manually by sending signal USR1"
 			" (kill -USR1 pid_atop)\n");
-	printf("or with the keystroke 't' in interactive mode.\n");
+	printf("or with the keystroke '%c' in interactive mode.\n", MSAMPNEXT);
 
 
 	cleanstop(1);
