@@ -82,6 +82,8 @@ int		compcpu(const void *, const void *);
 int		compdsk(const void *, const void *);
 int		compmem(const void *, const void *);
 int		compnet(const void *, const void *);
+int		compusr(const void *, const void *);
+int		compnam(const void *, const void *);
 
 int		cpucompar (const void *, const void *);
 int		diskcompar(const void *, const void *);
