@@ -6,11 +6,13 @@
 **
 ** This source-file contains functions to calculate the differences for
 ** the system-level and process-level counters since the previous sample.
-** ================================================================
+** ==========================================================================
 ** Author:      Gerlof Langeveld - AT Computing, Nijmegen, Holland
 ** E-mail:      gerlof@ATComputing.nl
 ** Date:        November 1996
 ** LINUX-port:  June 2000
+** --------------------------------------------------------------------------
+** Copyright (C) 2000-2005 Gerlof Langeveld
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -22,7 +24,18 @@
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 ** See the GNU General Public License for more details.
 **
+** You should have received a copy of the GNU General Public License
+** along with this program; if not, write to the Free Software
+** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+** --------------------------------------------------------------------------
+**
 ** $Log: deviate.c,v $
+** Revision 1.23  2005/10/31 12:45:29  gerlof
+** Support account-record version 3 (used by Mandriva).
+**
+** Revision 1.22  2005/10/21 09:49:38  gerlof
+** Per-user accumulation of resource consumption.
+**
 ** Revision 1.21  2004/12/14 15:05:47  gerlof
 ** Implementation of patch-recognition for disk and network-statistics.
 **
@@ -88,7 +101,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: deviate.c,v 1.21 2004/12/14 15:05:47 gerlof Exp $";
+static const char rcsid[] = "$Id: deviate.c,v 1.23 2005/10/31 12:45:29 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -270,7 +283,7 @@ deviatproc(struct pstat *aproc, int npresent,
 		*/
 		curstat = eproc+c;
 
-		if (kernpatch & PATCHACCT)
+		if (curstat->gen.pid)	/* acctrecord contains pid? */
 		{
 			if ( pdb_getproc(curstat->gen.pid, &pinfo) )
 					prestat = pinfo->pstat;
@@ -307,13 +320,11 @@ deviatproc(struct pstat *aproc, int npresent,
 
 		devstat->gen        = curstat->gen;
 
-		if ( !(kernpatch & PATCHACCT) )
-		{
+		if ( curstat->gen.pid == 0 )
 			devstat->gen.pid    = prestat.gen.pid;
 
-			if (!prestat.gen.pid)
-				devstat->gen.excode |= ~(LONG_MAX);
-		}
+		if (!prestat.gen.pid)
+			devstat->gen.excode |= ~(LONG_MAX);
 
 		strcpy(devstat->gen.cmdline, prestat.gen.cmdline);
 
