@@ -7,23 +7,33 @@
 ** This source-file contains functions to read the process-administration
 ** of every running process from kernel-space and extract the required
 ** activity-counters.
-** ================================================================
+** ==========================================================================
 ** Author:      Gerlof Langeveld - AT Computing, Nijmegen, Holland
 ** E-mail:      gerlof@ATComputing.nl
 ** Date:        November 1996
 ** LINUX-port:  June 2000
-** 
+** --------------------------------------------------------------------------
+** Copyright (C) 2000-2005 Gerlof Langeveld
+**
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
 ** Free Software Foundation; either version 2, or (at your option) any
 ** later version.
-** 
+**
 ** This program is distributed in the hope that it will be useful, but
 ** WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 ** See the GNU General Public License for more details.
 **
+** You should have received a copy of the GNU General Public License
+** along with this program; if not, write to the Free Software
+** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+** --------------------------------------------------------------------------
+**
 ** $Log: photoproc.c,v $
+** Revision 1.23  2005/10/21 09:49:57  gerlof
+** Per-user accumulation of resource consumption.
+**
 ** Revision 1.22  2004/12/14 15:05:58  gerlof
 ** Implementation of patch-recognition for disk and network-statistics.
 **
@@ -95,7 +105,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: photoproc.c,v 1.22 2004/12/14 15:05:58 gerlof Exp $";
+static const char rcsid[] = "$Id: photoproc.c,v 1.23 2005/10/21 09:49:57 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -130,12 +140,10 @@ photoproc(struct pstat *proclist, int maxproc)
 	register struct pstat	*curproc;
 
 	FILE		*fp;
-	DIR		*dirp, *dirtaskp;
-	struct dirent	*entp, *enttaskp;
-	int		nr, i;
+	DIR		*dirp;
+	struct dirent	*entp;
 	char		origdir[1024];
-	int		nthreads;
-	int		pval=0;
+	int		nr, i, nthreads, pval=0;
 
 	/*
 	** one-time initialization stuff
@@ -232,14 +240,21 @@ photoproc(struct pstat *proclist, int maxproc)
 			}
 		}
 
+#if	0
 		/*
 		** Linux >= 2.6
 		** For multithreaded processes, the subdirectory 'task'
 		** contains "pid's" again for the threads related to
 		** this process.
+		**
+		** This piece of code is not valid any more due to modified
+		** accounting by the kernel.
 		*/ 
 		if ( nthreads > 1 && chdir("task") == 0 )
 		{
+			DIR		*dirtaskp;
+			struct dirent	*enttaskp;
+
 			dirtaskp = opendir(".");
 
 			while ( (enttaskp = readdir(dirtaskp))
@@ -251,13 +266,6 @@ photoproc(struct pstat *proclist, int maxproc)
 				if (!isdigit(enttaskp->d_name[0]))
 					continue;
 
-				/*
-				** skip the tgroup-leader (we know
-				** that info already)
-				*/
-				if (strcmp(entp->d_name, enttaskp->d_name)==0)
-					continue;
-
 				/*
 				** change to the underlying process' subdir
 				*/
@@ -279,6 +287,7 @@ photoproc(struct pstat *proclist, int maxproc)
 			closedir(dirtaskp);
 			chdir("..");
 		}
+#endif
 
 		chdir("..");
 	}
