@@ -6,23 +6,44 @@
 ** 
 ** This source-file contains the print-functions to visualize the calculated
 ** figures.
-** ================================================================
+** ==========================================================================
 ** Author:      Gerlof Langeveld - AT Computing, Nijmegen, Holland
 ** E-mail:      gerlof@ATComputing.nl
 ** Date:        November 1996
 ** LINUX-port:  June 2000
-** 
+** --------------------------------------------------------------------------
+** Copyright (C) 2000-2005 Gerlof Langeveld
+**
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
 ** Free Software Foundation; either version 2, or (at your option) any
 ** later version.
-** 
+**
 ** This program is distributed in the hope that it will be useful, but
 ** WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 ** See the GNU General Public License for more details.
 **
+** You should have received a copy of the GNU General Public License
+** along with this program; if not, write to the Free Software
+** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+** --------------------------------------------------------------------------
+** 
 ** $Log: showgeneric.c,v $
+** Revision 1.39  2005/11/04 14:16:16  gerlof
+** Minor bug-solutions.
+**
+** Revision 1.38  2005/10/28 09:52:03  gerlof
+** All flags/subcommands are defined as macro's.
+** Subcommand 'p' has been changed to 'z' (pause).
+**
+** Revision 1.37  2005/10/24 06:12:17  gerlof
+** Flag -L modified into -l.
+**
+** Revision 1.36  2005/10/21 09:50:46  gerlof
+** Per-user accumulation of resource consumption.
+** Possibility to send signal to process.
+**
 ** Revision 1.35  2004/12/14 15:06:41  gerlof
 ** Implementation of patch-recognition for disk and network-statistics.
 **
@@ -131,7 +152,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: showgeneric.c,v 1.35 2004/12/14 15:06:41 gerlof Exp $";
+static const char rcsid[] = "$Id: showgeneric.c,v 1.39 2005/11/04 14:16:16 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -174,7 +195,7 @@ static int	paused;     	/* boolean: currently in pause-mode     */
 static int	fixedhead;	/* boolean: fixate header-lines         */
 
 static int	(*curcompar)(const void *, const void *) = compcpu;
-static char	*sortorder = "CPU", showtype = 'g';
+static char	*sortorder = "CPU", showtype = MPROCGEN;
 
 static int	maxcpulines = 999;  /* maximum cpu       lines          */
 static int	maxdsklines = 999;  /* maximum disk      lines          */
@@ -183,6 +204,8 @@ static int	maxintlines = 999;  /* maximum interface lines          */
 char    	*genhdr = "ATOP - %-18.18s "
                           "%s  %s     %12ld seconds elapsed\n";
 
+static int	cumusers(struct pstat *, struct pstat *, int);
+static int	cumprocs(struct pstat *, struct pstat *, int);
 static long	getnumval(char *, long, int);
 
 /*
@@ -196,12 +219,15 @@ generic_samp(time_t curtime, int nsecs,
 	static int	callnr = 0;
 
 	register int	i, curline=1, statline;
-	int		firstproc, plistsz;
+	int		firstproc, plistsz, alistsz, killpid, killsig;
 	int		lastchar;
 	char		format1[16], format2[16];
 	struct passwd 	*pwd;
 	struct syscap	syscap;
 
+	struct pstat	*save_pstat = NULL;
+	int		 save_nact = 0;
+
 	if (callnr == 0)	/* first call ? */
 	{
 		/*
@@ -212,45 +238,45 @@ generic_samp(time_t curtime, int nsecs,
 		{
 			switch (flaglist[i])
 			{
-			   case 'C':
+			   case MSORTCPU:
 				curcompar = compcpu;
 				sortorder = "CPU";
 				break;
 
-			   case 'M':
+			   case MSORTMEM:
 				curcompar = compmem;
 				sortorder = "MEM";
 				break;
 
-			   case 'D':
+			   case MSORTDSK:
 				curcompar = compdsk;
 				sortorder = "DSK";
 				break;
 
-			   case 'N':
+			   case MSORTNET:
 				curcompar = compnet;
 				sortorder = "NET";
 				break;
 
-			   case 'g':
-				showtype  = 'g';
+			   case MPROCGEN:
+				showtype  = MPROCGEN;
 				curcompar = compcpu;
 				sortorder = "CPU";
 				break;
 
-			   case 'm':
-				showtype  = 'm';
+			   case MPROCMEM:
+				showtype  = MPROCMEM;
 				curcompar = compmem;
 				sortorder = "MEM";
 				break;
 
-			   case 's':
-				showtype  = 's';
+			   case MPROCSCH:
+				showtype  = MPROCSCH;
 				curcompar = compcpu;
 				sortorder = "CPU";
 				break;
 
-			   case 'd':
+			   case MPROCDSK:
 				if ( !(kernpatch & PATCHSTAT) )
 				{
 					fprintf(stderr,
@@ -260,12 +286,12 @@ generic_samp(time_t curtime, int nsecs,
 					break;
 				}
 
-				showtype  = 'd';
+				showtype  = MPROCDSK;
 				curcompar = compdsk;
 				sortorder = "DSK";
 				break;
 
-			   case 'n':
+			   case MPROCNET:
 				if ( !(kernpatch & PATCHSTAT) )
 				{
 					fprintf(stderr,
@@ -275,33 +301,41 @@ generic_samp(time_t curtime, int nsecs,
 					break;
 				}
 
-				showtype  = 'n';
+				showtype  = MPROCNET;
 				curcompar = compnet;
 				sortorder = "NET";
 				break;
 
-			   case 'v':
-				showtype  = 'v';
+			   case MPROCVAR:
+				showtype  = MPROCVAR;
 				break;
 
-			   case 'c':
-				showtype  = 'c';
+			   case MPROCARG:
+				showtype  = MPROCARG;
 				break;
 
-			   case 'f':
+			   case MCUMUSER:
+				showtype  = MCUMUSER;
+				break;
+
+			   case MCUMPROC:
+				showtype  = MCUMPROC;
+				break;
+
+			   case MSYSFIXED:
 				if (fixedhead)
 					fixedhead=0;
 				else
 					fixedhead=1;
 				break;
 
-			   case 'l':
+			   case MSYSLIMIT:
 				maxcpulines = 0;
 				maxdsklines = 5;
 				maxintlines = 3;
 				break;
 
-			   case 'V':
+			   case MVERSION:
 				printf("%s\n", getversion());
 				cleanstop(0);
 
@@ -455,7 +489,46 @@ generic_samp(time_t curtime, int nsecs,
 	while (1)
 	{
 		/*
-		** sort the list of active processes in required order 
+		** if cumulative figures required, save the current
+		** list of processes and accumulate resource consumption
+		** of all processes in the current list
+		*/
+		if (showtype == MCUMUSER || showtype == MCUMPROC)
+		{
+			/*
+			** remember info based on individual processes
+			*/
+			save_pstat = pstat;
+			save_nact  = nact;
+
+			/*
+			** allocate space for new (temporary) list with
+			** one entry per user/program (list has worst-case size)
+			*/
+			pstat = calloc(sizeof(struct pstat), nact);
+
+			/*
+			** accumulate all processes
+			** return-value nact contains number of new entries
+			*/ 
+			switch (showtype)
+			{
+			   case MCUMUSER:
+				nact = cumusers(save_pstat, pstat, save_nact);
+				break;
+			   case MCUMPROC:
+				nact = cumprocs(save_pstat, pstat, save_nact);
+				break;
+			}
+
+			if (screen)
+				plistsz = LINES-curline-2;
+			else
+				plistsz = nact;
+		}
+
+		/*
+		** sort the list in required order 
 		** (default CPU-consumption) and print the list
 		*/
 		if (nact > 0 && plistsz > 0)
@@ -486,6 +559,20 @@ generic_samp(time_t curtime, int nsecs,
 				&procsel, screen);
 		}
 
+		alistsz = nact;		/* preserve size of active list */
+
+		/*
+		** if cumulative figures per user shown,
+		** release temporary space and restore per-process values
+		*/
+		if (showtype == MCUMUSER || showtype == MCUMPROC)
+		{
+			free(pstat);
+
+			pstat = save_pstat;
+			nact  = save_nact;
+		}
+
 		/*
 		** in case of writing to a terminal, the user can also enter
 		** a character to switch options, etc
@@ -520,7 +607,7 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** stop it
 			   */
-			   case 'q':
+			   case MQUIT:
 				move(LINES-1, 0);
 				clrtoeol();
 				refresh();
@@ -529,7 +616,7 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** manual trigger for next sample
 			   */
-			   case 't':
+			   case MSAMPNEXT:
 				if (paused)
 					break;
 
@@ -539,7 +626,7 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** manual trigger for previous sample
 			   */
-			   case 'T':
+			   case MSAMPPREV:
 				if (!rawreadflag)
 				{
 					printw("Only allowed when viewing "
@@ -556,7 +643,7 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** sort in cpu-activity order
 			   */
-			   case 'C':
+			   case MSORTCPU:
 				curcompar = compcpu;
 				sortorder = "CPU";
 				firstproc = 0;
@@ -565,7 +652,7 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** sort in memory-consumption order
 			   */
-			   case 'M':
+			   case MSORTMEM:
 				curcompar = compmem;
 				sortorder = "MEM";
 				firstproc = 0;
@@ -574,7 +661,7 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** sort in disk-activity order
 			   */
-			   case 'D':
+			   case MSORTDSK:
 				curcompar = compdsk;
 				sortorder = "DSK";
 				firstproc = 0;
@@ -583,7 +670,7 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** sort in network-activity order
 			   */
-			   case 'N':
+			   case MSORTNET:
 				curcompar = compnet;
 				sortorder = "NET";
 				firstproc = 0;
@@ -592,8 +679,8 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** general figures per process
 			   */
-			   case 'g':
-				showtype  = 'g';
+			   case MPROCGEN:
+				showtype  = MPROCGEN;
 				curcompar = compcpu;
 				sortorder = "CPU";
 				firstproc = 0;
@@ -602,8 +689,8 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** memory-specific figures per process
 			   */
-			   case 'm':
-				showtype  = 'm';
+			   case MPROCMEM:
+				showtype  = MPROCMEM;
 				curcompar = compmem;
 				sortorder = "MEM";
 				firstproc = 0;
@@ -612,14 +699,14 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** disk-specific figures per process
 			   */
-			   case 'd':
+			   case MPROCDSK:
 				if ( !(kernpatch & PATCHSTAT) )
 				{
 					printw("No kernel-patch installed; "
 					       "request ignored!");
 					break;
 				}
-				showtype  = 'd';
+				showtype  = MPROCDSK;
 				curcompar = compdsk;
 				sortorder = "DSK";
 				firstproc = 0;
@@ -628,14 +715,14 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** network-specific figures per process
 			   */
-			   case 'n':
+			   case MPROCNET:
 				if ( !(kernpatch & PATCHSTAT) )
 				{
 					printw("No kernel-patch installed; "
 					       "request ignored!");
 					break;
 				}
-				showtype  = 'n';
+				showtype  = MPROCNET;
 				curcompar = compnet;
 				sortorder = "NET";
 				firstproc = 0;
@@ -644,35 +731,58 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** various info per process
 			   */
-			   case 'v':
-				showtype  = 'v';
+			   case MPROCVAR:
+				showtype  = MPROCVAR;
 				firstproc = 0;
 				break;
 
 			   /*
 			   ** command-line per process
 			   */
-			   case 'c':
-				showtype  = 'c';
+			   case MPROCARG:
+				showtype  = MPROCARG;
 				firstproc = 0;
 				break;
 
 			   /*
 			   ** scheduling-values per process
 			   */
-			   case 's':
-				showtype  = 's';
+			   case MPROCSCH:
+				showtype  = MPROCSCH;
 				curcompar = compcpu;
 				sortorder = "CPU";
 				firstproc = 0;
 				break;
 
+			   /*
+			   ** accumulated resource consumption per user
+			   */
+			   case MCUMUSER:
+				printw("Consumption per user; use '%c' to "
+				       "toggle between all/active processes",
+				       MALLPROC);
+
+				showtype  = MCUMUSER;
+				firstproc = 0;
+				break;
+
+			   /*
+			   ** accumulated resource consumption per program
+			   */
+			   case MCUMPROC:
+				printw("Consumption per command; use '%c' to "
+				       "toggle between all/active processes",
+				       MALLPROC);
+
+				showtype  = MCUMPROC;
+				firstproc = 0;
+				break;
+
 			   /*
 			   ** help wanted?
 			   */
-			   case '?':
-			   case 'h':
-			   case 'H':
+			   case MHELP1:
+			   case MHELP2:
 				alarm(0);	/* stop the clock         */
 
 				move(statline, 0);
@@ -689,10 +799,63 @@ generic_samp(time_t curtime, int nsecs,
 				firstproc = 0;
 				break;
 
+			   /*
+			   ** send signal to process
+			   */
+			   case MKILLPROC:
+				if (rawreadflag)
+				{
+					printw("Not possible when viewing "
+					       "raw file!");
+					beep();
+					break;
+				}
+
+				alarm(0);	/* stop the clock */
+
+				killpid = getnumval("Pid of process: ",
+						     0, statline);
+
+				switch (killpid)
+				{
+				   case 0:
+				   case -1:
+					break;
+
+				   case 1:
+					printw("Sending signal to pid 1 not "
+					       "allowed!");
+					beep();
+					refresh();
+					sleep(3);	
+					break;
+
+				   default:
+					clrtoeol();
+					killsig = getnumval("Signal [%d]: ",
+						     15, statline);
+
+					if ( kill(killpid, killsig) == -1)
+					{
+						printw("Not possible to send "
+					               "signal %d to pid %d!",
+							killsig, killpid);
+						beep();
+						refresh();
+						sleep(3);	
+					}
+				}
+
+				if (!paused)
+					alarm(1); /* set short timer */
+
+				firstproc = 0;
+				break;
+
 			   /*
 			   ** change interval timeout
 			   */
-			   case 'i':
+			   case MINTERVAL:
 				if (rawreadflag)
 				{
 					printw("Not possible when viewing "
@@ -716,7 +879,8 @@ generic_samp(time_t curtime, int nsecs,
 				{
 					clrtoeol();
 					printw("No timer set; waiting for "
-					       "manual trigger ('t').....");
+					       "manual trigger ('%c').....",
+						MSAMPNEXT);
 					refresh();
 				}
 
@@ -726,13 +890,13 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** focus on specific user
 			   */
-			   case 'u':
+			   case MSELUSER:
 				alarm(0);	/* stop the clock */
 				echo();
 
 				move(statline, 0);
 				clrtoeol();
-				printw("User-name as regular expression "
+				printw("Username as regular expression "
 				       "(enter=all users): ");
 
 				procsel.username[0] = '\0';
@@ -792,7 +956,7 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** focus on specific process-name
 			   */
-			   case 'P':
+			   case MSELPROC:
 				alarm(0);	/* stop the clock */
 				echo();
 
@@ -824,7 +988,7 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** toggle pause-state
 			   */
-			   case 'p':
+			   case MPAUSE:
 				if (paused)
 				{
 					paused=0;
@@ -847,7 +1011,7 @@ generic_samp(time_t curtime, int nsecs,
 			   ** toggle between modified processes and
 			   ** all processes
 			   */
-			   case 'a':
+			   case MALLPROC:
 				if (rawreadflag)
 				{
 					printw("Process-list from raw file "
@@ -858,14 +1022,14 @@ generic_samp(time_t curtime, int nsecs,
 				if (deviatonly)
 				{
 					deviatonly=0;
-					printw("All processes will be shown "
-					       "......");
+					printw("All processes will be "
+					       "shown/accumulated......");
 				}
 				else
 				{
 					deviatonly=1;
 					printw("Only active processes will"
-					       " be shown.....");
+					       " be shown/accumulated.....");
 				}
 
 				if (interval && !paused && !rawreadflag)
@@ -878,7 +1042,7 @@ generic_samp(time_t curtime, int nsecs,
 			   ** system-statistics lines:
 			   **	         toggle fixed or variable
 			   */
-			   case 'f':
+			   case MSYSFIXED:
 				if (fixedhead)
 				{
 					fixedhead=0;
@@ -899,7 +1063,7 @@ generic_samp(time_t curtime, int nsecs,
 			   ** system-statistics lines:
 			   **	         toggle no or all active disk
 			   */
-			   case 'l':
+			   case MSYSLIMIT:
 				alarm(0);	/* stop the clock */
 
 				maxcpulines =
@@ -926,7 +1090,7 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** reset statistics 
 			   */
-			   case 'r':
+			   case MRESET:
 				getalarm(0);	/* restart the clock */
 				paused = 0;
 				return lastchar;
@@ -934,22 +1098,22 @@ generic_samp(time_t curtime, int nsecs,
 			   /*
 			   ** show version info
 			   */
-			   case 'V':
+			   case MVERSION:
 				printw("%s", getversion());
 				break;
 
 			   /*
 			   ** handle forward
 			   */
-			   case 0x06:
-				if (nact-firstproc > plistsz)
+			   case MLISTFW:
+				if (alistsz-firstproc > plistsz)
 					firstproc += plistsz;
 				break;
 
 			   /*
 			   ** handle backward
 			   */
-			   case 0x02:
+			   case MLISTBW:
 				if (firstproc >= plistsz)
 					firstproc -= plistsz;
 				break;
@@ -970,10 +1134,134 @@ generic_samp(time_t curtime, int nsecs,
 			}
 		}
 		else	/* no screen */
+		{
 			return '\0';
+		}
 	}
 }
 
+/*
+** accumulate all processes per user in new list
+*/
+static int
+cumusers(struct pstat *curprocs, struct pstat *curusers, int numprocs)
+{
+	register int	i, numusers;
+
+	/*
+	** sort list of active processes in order of uid (increasing)
+	*/
+	qsort(curprocs, numprocs, sizeof(struct pstat), compusr);
+
+	/*
+	** accumulate all processes per user in the new list
+	*/
+	for (numusers=i=0; i < numprocs; i++, curprocs++)
+	{
+		if ( curusers->gen.ruid != curprocs->gen.ruid )
+		{
+			if (curusers->gen.pid)
+			{
+				numusers++;
+				curusers++;
+			}
+			curusers->gen.ruid = curprocs->gen.ruid;
+		}
+
+		curusers->gen.pid++;		/* misuse as counter */
+
+		curusers->gen.nthr   += curprocs->gen.nthr;
+		curusers->cpu.utime  += curprocs->cpu.utime;
+		curusers->cpu.stime  += curprocs->cpu.stime;
+
+		curusers->dsk.rio    += curprocs->dsk.rio;
+		curusers->dsk.wio    += curprocs->dsk.wio;
+			
+		curusers->net.tcpsnd += curprocs->net.tcpsnd;
+		curusers->net.tcprcv += curprocs->net.tcprcv;
+		curusers->net.udpsnd += curprocs->net.udpsnd;
+		curusers->net.udprcv += curprocs->net.udprcv;
+		curusers->net.rawsnd += curprocs->net.rawsnd;
+		curusers->net.rawrcv += curprocs->net.rawrcv;
+
+		if (curprocs->gen.state != 'E')
+		{
+			curusers->mem.rmem   += curprocs->mem.rmem;
+			curusers->mem.vmem   += curprocs->mem.vmem;
+			curusers->mem.rgrow  += curprocs->mem.rgrow;
+			curusers->mem.vgrow  += curprocs->mem.vgrow;
+		}
+	}
+
+	if (curusers->gen.pid)
+		numusers++;
+
+	return numusers;
+}
+
+/*
+** accumulate all processes with the same name (i.e. same program)
+** into a new list
+*/
+static int
+cumprocs(struct pstat *curprocs, struct pstat *curprogs, int numprocs)
+{
+	register int	i, numprogs;
+
+	/*
+	** sort list of active processes in order of process-name
+	*/
+	qsort(curprocs, numprocs, sizeof(struct pstat), compnam);
+
+	/*
+	** accumulate all processes with same name in the new list
+	*/
+	for (numprogs=i=0; i < numprocs; i++, curprocs++)
+	{
+		if ( strcmp(curprogs->gen.name, curprocs->gen.name) != 0)
+		{
+			if (curprogs->gen.pid)
+			{
+				numprogs++;
+				curprogs++;
+			}
+			strcpy(curprogs->gen.name, curprocs->gen.name);
+		}
+
+		curprogs->gen.pid++;		/* misuse as counter */
+
+		curprogs->gen.nthr   += curprocs->gen.nthr;
+		curprogs->cpu.utime  += curprocs->cpu.utime;
+		curprogs->cpu.stime  += curprocs->cpu.stime;
+
+		curprogs->dsk.rio    += curprocs->dsk.rio;
+		curprogs->dsk.wio    += curprocs->dsk.wio;
+			
+		curprogs->net.tcpsnd += curprocs->net.tcpsnd;
+		curprogs->net.tcprcv += curprocs->net.tcprcv;
+		curprogs->net.udpsnd += curprocs->net.udpsnd;
+		curprogs->net.udprcv += curprocs->net.udprcv;
+		curprogs->net.rawsnd += curprocs->net.rawsnd;
+		curprogs->net.rawrcv += curprocs->net.rawrcv;
+
+		if (curprocs->gen.state != 'E')
+		{
+			curprogs->mem.rmem   += curprocs->mem.rmem;
+			curprogs->mem.vmem   += curprocs->mem.vmem;
+			curprogs->mem.rgrow  += curprocs->mem.rgrow;
+			curprogs->mem.vgrow  += curprocs->mem.vgrow;
+		}
+	}
+
+	if (curprogs->gen.pid)
+		numprogs++;
+
+	return numprogs;
+}
+
+/*
+** get a numerical value from the user and verify 
+*/
 static long
 getnumval(char *ask, long valuenow, int statline)
 {
@@ -1037,47 +1325,65 @@ printg(const char *format, ...)
 /*
 ** show help information in interactive mode
 */
-static char *helptext[] = {
-	"Figures shown for active processes:",
-	"\t'g' - generic info (default)",
-	"\t'm' - memory details",
-	"\t'd' - disk details",
-	"\t'n' - network details",
-	"\t's' - scheduling and thread-group info",
-	"\t'v' - various info (user/group, date/time)",
-	"\t'c' - full command-line per process",
-	"",
-	"Sort list of active processes by:",
-	"\t'C' - cpu activity (default)",
-	"\t'M' - memory consumption",
-	"\t'D' - disk activity",
-	"\t'N' - network activity",
-	"",
-	"Screen-handling:",
-	"\t^F  - show next     page in the process-list (forward)",
-	"\t^B  - show previous page in the process-list (backward)",
-	"",
-	"Miscellaneous commands:",
-	"\t'i' - change interval-timer (0 = only manual trigger)",
-	"\t't' - manual trigger to finish interval",
-	"\t'T' - show previous interval again (raw file viewing)",
-	"\t'r' - reset counters to zero (or rewind for raw file viewing)",
-	"",
-	"\t'u' - focus on specific user-name string    (regular expression)",
-	"\t'P' - focus on specific process-name string (regular expression)",
-	"\t'a' - active processes only (default) or all processes (toggle)",
-	"\t'p' - pause-button                                     (toggle)",
-	"\t'f' - fixate on static range of header-lines           (toggle)",
-	"",
-	"\t'l' - limited lines for per-cpu, disk and interface resources",
-	"",
-	"\t'V' - version-information",
-	"\t'h' - help-information",
-	"\t'?' - help-information",
-	"\t'q' - quit this program",
+static struct helptext {
+	char *helpline;
+	char helparg;
+} helptext[] = {
+	{"Figures shown for active processes:\n", 		' '},
+	{"\t'%c' - generic info (default)\n",			MPROCGEN},
+	{"\t'%c' - memory details\n",				MPROCMEM},
+	{"\t'%c' - disk details\n",				MPROCDSK},
+	{"\t'%c' - network details\n",				MPROCNET},
+	{"\t'%c' - scheduling and thread-group info\n",		MPROCSCH},
+	{"\t'%c' - various info (user/group, date/time)\n",	MPROCVAR},
+	{"\t'%c' - full command-line per process\n",		MPROCARG},
+	{"\n",							' '},
+	{"Accumulated figures:\n",				' '},
+	{"\t'%c' - total resource consumption per user\n", 	MCUMUSER},
+	{"\t'%c' - total resource consumption per program (i.e. same "
+	 "process name)\n",					MCUMPROC},
+	{"\n",							' '},
+	{"Sort list of active processes by:\n",			' '},
+	{"\t'%c' - cpu activity (default)\n",			MSORTCPU},
+	{"\t'%c' - memory consumption\n",			MSORTMEM},
+	{"\t'%c' - disk activity\n",				MSORTDSK},
+	{"\t'%c' - network activity\n",				MSORTNET},
+	{"\n",							' '},
+	{"Screen-handling:\n",					' '},
+	{"\t^F  - show next     page in the process-list (forward)\n",	' '},
+	{"\t^B  - show previous page in the process-list (backward)\n", ' '},
+	{"\n",							' '},
+	{"Miscellaneous commands:\n",				' '},
+	{"\t'%c' - change interval-timer (0 = only manual trigger)\n",
+								MINTERVAL},
+	{"\t'%c' - manual trigger to finish interval\n",	MSAMPNEXT},
+	{"\t'%c' - show previous interval again (raw file viewing)\n",
+								MSAMPPREV},
+	{"\t'%c' - reset counters to zero (or rewind for raw file viewing)\n",
+								MRESET},
+	{"\n",							' '},
+	{"\t'%c' - focus on specific user name    (regular expression)\n",
+								MSELUSER},
+	{"\t'%c' - focus on specific process name (regular expression)\n",
+								MSELPROC},
+	{"\n",							' '},
+	{"\t'%c' - active processes only (default) or all processes (toggle)\n",
+								MALLPROC},
+	{"\t'%c' - pause-button to freeze current sample            (toggle)\n",
+								MPAUSE},
+	{"\t'%c' - fixate on static range of header-lines           (toggle)\n",
+								MSYSFIXED},
+	{"\t'%c' - limited lines for per-cpu, disk and interface resources\n",
+								MSYSLIMIT},
+	{"\t'%c' - kill a process (i.e. send a signal)\n",	MKILLPROC},
+	{"\n",							' '},
+	{"\t'%c' - version-information\n",			MVERSION},
+	{"\t'%c' - help-information\n",				MHELP1},
+	{"\t'%c' - help-information\n",				MHELP2},
+	{"\t'%c' - quit this program\n",			MQUIT},
 };
 
-static int helplines = sizeof(helptext)/sizeof(char *);
+static int helplines = sizeof(helptext)/sizeof(struct helptext);
 
 static void
 showhelp(int helpline)
@@ -1097,7 +1403,7 @@ showhelp(int helpline)
 	*/
 	for (lin=0; lin < helplines; lin++)
 	{
-		wprintw(helpwin, "%s\n", helptext[lin]);
+		wprintw(helpwin, helptext[lin].helpline, helptext[lin].helparg);
 
 		/*
 		** when the window is full, start paging interactively
@@ -1154,19 +1460,36 @@ generic_end(void)
 void
 generic_usage(void)
 {
-	printf("\t  -g  show general process-info (default)\n");
-	printf("\t  -m  show memory-related process-info\n");
-	printf("\t  -d  show disk-related process-info\n");
-	printf("\t  -n  show network-related process-info\n");
-	printf("\t  -s  show scheduling-related process-info\n");
-	printf("\t  -v  show various process-info (user/group, date/time)\n");
-	printf("\t  -c  show command-line per process\n");
-	printf("\t  -C  sort by cpu-consumption (default)\n");
-	printf("\t  -M  sort by memory-consumption\n");
-	printf("\t  -D  sort by disk-activity\n");
-	printf("\t  -N  sort by network-activity\n");
-	printf("\t  -f  show fixed number of system-statistics\n");
-	printf("\t  -l  limited lines for system-resources\n");
+	printf("\t  -%c  show general process-info (default)\n",
+			MPROCGEN);
+	printf("\t  -%c  show memory-related process-info\n",
+			MPROCMEM);
+	printf("\t  -%c  show disk-related process-info\n",
+			MPROCDSK);
+	printf("\t  -%c  show network-related process-info\n",
+			MPROCNET);
+	printf("\t  -%c  show scheduling-related process-info\n",
+			MPROCSCH);
+	printf("\t  -%c  show various process-info (user/group, date/time)\n",
+			MPROCVAR);
+	printf("\t  -%c  show command-line per process\n",
+			MPROCARG);
+	printf("\t  -%c  show totals per user\n",
+			MCUMUSER);
+	printf("\t  -%c  show totals per program (i.e. same process name)\n",
+			MCUMPROC);
+	printf("\t  -%c  sort by cpu-consumption (default)\n",
+			MSORTCPU);
+	printf("\t  -%c  sort by memory-consumption\n",
+			MSORTMEM);
+	printf("\t  -%c  sort by disk-activity\n",
+			MSORTDSK);
+	printf("\t  -%c  sort by network-activity\n",
+			MSORTNET);
+	printf("\t  -%c  show fixed number of system-statistics\n",
+			MSYSFIXED);
+	printf("\t  -%c  limited lines for system-resources\n",
+			MSYSLIMIT);
 }
 
 /*
@@ -1280,69 +1603,77 @@ do_flags(char *val)
 		   case '-':
 			break;
 
-		   case 'C':
+		   case MSORTCPU:
 			curcompar = compcpu;
 			sortorder = "CPU";
 			break;
 
-		   case 'M':
+		   case MSORTMEM:
 			curcompar = compmem;
 			sortorder = "MEM";
 			break;
 
-		   case 'D':
+		   case MSORTDSK:
 			curcompar = compdsk;
 			sortorder = "DSK";
 			break;
 
-		   case 'N':
+		   case MSORTNET:
 			curcompar = compnet;
 			sortorder = "NET";
 			break;
 
-		   case 'g':
-			showtype  = 'g';
+		   case MPROCGEN:
+			showtype  = MPROCGEN;
 			curcompar = compcpu;
 			sortorder = "CPU";
 			break;
 
-		   case 'm':
-			showtype  = 'm';
+		   case MPROCMEM:
+			showtype  = MPROCMEM;
 			curcompar = compmem;
 			sortorder = "MEM";
 			break;
 
-		   case 'd':
-			showtype  = 'd';
+		   case MPROCDSK:
+			showtype  = MPROCDSK;
 			curcompar = compdsk;
 			sortorder = "DSK";
 			break;
 
-		   case 'n':
-			showtype  = 'n';
+		   case MPROCNET:
+			showtype  = MPROCNET;
 			curcompar = compnet;
 			sortorder = "NET";
 			break;
 
-		   case 'v':
-			showtype  = 'v';
+		   case MPROCVAR:
+			showtype  = MPROCVAR;
 			break;
 
-		   case 's':
-			showtype  = 's';
+		   case MPROCSCH:
+			showtype  = MPROCSCH;
 			curcompar = compcpu;
 			sortorder = "CPU";
 			break;
 
-		   case 'c':
-			showtype  = 'c';
+		   case MPROCARG:
+			showtype  = MPROCARG;
 			break;
 
-		   case 'a':
+		   case MCUMUSER:
+			showtype  = MCUMUSER;
+			break;
+
+		   case MCUMPROC:
+			showtype  = MCUMPROC;
+			break;
+
+		   case MALLPROC:
 			deviatonly  = 0;
 			break;
 
-		   case 'f':
+		   case MSYSFIXED:
 			fixedhead=1;
 			break;
 		}
