@@ -25,6 +25,7 @@
 #define MAXUSERSEL	64
 
 struct syscap {
+	int	nrcpu;
 	count_t	availcpu;
 	count_t	availmem;
 	count_t	availdsk;
@@ -40,6 +41,48 @@ struct selection {
 	regex_t	procregex;
 };
 
+/*
+** list with keystrokes/flags
+*/
+#define	MPROCGEN	'g'
+#define	MPROCMEM	'm'
+#define	MPROCDSK	'd'
+#define	MPROCNET	'n'
+#define	MPROCSCH	's'
+#define	MPROCVAR	'v'
+#define	MPROCARG	'c'
+
+#define	MCUMUSER	'u'
+#define	MCUMPROC	'p'
+
+#define	MSORTCPU	'C'
+#define	MSORTDSK	'D'
+#define	MSORTMEM	'M'
+#define	MSORTNET	'N'
+
+#define	MSYSFIXED	'f'
+#define	MSYSLIMIT	'l'
+
+#define	MSELPROC	'P'
+#define	MSELUSER	'U'
+
+#define	MALLPROC	'a'
+#define	MKILLPROC	'k'
+#define	MLISTFW		0x06
+#define	MLISTBW		0x02
+#define	MINTERVAL	'i'
+#define	MPAUSE		'z'
+#define	MQUIT		'q'
+#define	MRESET		'r'
+#define	MSAMPNEXT	't'
+#define	MSAMPPREV	'T'
+#define	MVERSION	'V'
+#define	MHELP1		'?'
+#define	MHELP2		'h'
+
+/*
+** general function prototypes
+*/
 void	totalcap   (struct syscap *, struct sstat *, struct pstat *, int);
 void	pricumproc (struct pstat *, int, int, int, int);
 void	showgenproc(struct pstat *, double);
