@@ -6,22 +6,39 @@
 **
 ** This source-file contains the Linux-specific functions to calculate
 ** figures to be visualized.
-** ================================================================
+** ==========================================================================
 ** Author:      Gerlof Langeveld - AT Computing, Nijmegen, Holland
 ** E-mail:      gerlof@ATComputing.nl
 ** Date:        July 2002
-** 
+** --------------------------------------------------------------------------
+** Copyright (C) 2000-2005 Gerlof Langeveld
+**
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
 ** Free Software Foundation; either version 2, or (at your option) any
 ** later version.
-** 
+**
 ** This program is distributed in the hope that it will be useful, but
 ** WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 ** See the GNU General Public License for more details.
 **
+** You should have received a copy of the GNU General Public License
+** along with this program; if not, write to the Free Software
+** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+** --------------------------------------------------------------------------
+**
 ** $Log: showlinux.c,v $
+** Revision 1.27  2005/11/04 14:16:45  gerlof
+** Minor bug-solutions.
+**
+** Revision 1.26  2005/10/28 09:51:29  gerlof
+** All flags/subcommands are defined as macro's.
+** Subcommand 'p' has been changed to 'z' (pause).
+**
+** Revision 1.25  2005/10/21 09:51:11  gerlof
+** Per-user accumulation of resource consumption.
+**
 ** Revision 1.24  2004/12/14 15:06:48  gerlof
 ** Implementation of patch-recognition for disk and network-statistics.
 **
@@ -102,7 +119,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: showlinux.c,v 1.24 2004/12/14 15:06:48 gerlof Exp $";
+static const char rcsid[] = "$Id: showlinux.c,v 1.27 2005/11/04 14:16:45 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -139,12 +156,12 @@ extern int      osvers;
 char 	*prcline   	= "PRC | sys %s | user %s | #thr %s | #exits %s |"
 	       	     	  " #zombie %s |\n";
 /***************************************************************************/
-char	*genprochdr1	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW USERNAME NTHRD"
-		     	  " STATUS S  %s CMD  %4d/%-4d\n";
-char 	*genproclina1	= "%5d %s %s %s %s %-8.8s %3d/%c %-6s %c %3.0lf%% "
+char	*genprochdr1	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW USERNAME THR  "
+		     	  "ST EXC  S  %s CMD  %4d/%-4d\n";
+char 	*genproclina1	= "%5d %s %s %s %s %-8.8s %3d  %c%c      %c %3.0lf%% "
 	                  "%.14s\n";
-char 	*genprocline1	= "%s %s               %-8.8s       %s"
-			  " E %3.0lf%% <%.12s>\n";
+char 	*genprocline1	= "%s %s               %-8.8s      %c%c %3ld  "
+			  "E %3.0lf%% <%.12s>\n";
 
 char	*genprochdr2	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW RDDSK WRDSK "
 		     	  "RNET SNET S  %s CMD  %4d/%-4d\n";
@@ -179,22 +196,33 @@ char 	*netproclinep	= "%5d %s/%s %s/%s %s/%s %s/%s %2llu %2llu "
 			  "%3.0lf%% <%.12s>\n";
 /***************************************************************************/
 char	*varprochdr	= "\n  PID  USERNAME GROUP       STDATE    STTIME      "
-	                  "STATUS  S  %s CMD  %4d/%-4d\n";
-char 	*varproclina	= "%5d  %-8.8s %-8.8s  %s %s     %-6s  %c %3.0lf%% "
+	                  "ST EXC  S  %s CMD  %4d/%-4d\n";
+char 	*varproclina	= "%5d  %-8.8s %-8.8s  %s %s     %c%c      %c %3.0lf%% "
 	                  "%.14s\n";
-char 	*varprocline	= " %-8.8s %-8.8s  %s %s     %s  E %3.0lf%% <%.12s>\n";
+char 	*varprocline	= " %-8.8s %-8.8s  %s %s     %c%c %3ld  E %3.0lf%% "
+			  "<%.12s>\n";
 /***************************************************************************/
 char	*cmdprochdr	= "\n  PID  %s COMMAND-LINE                           "
 			  "                    %4d/%-4d\n";
 char 	*cmdproclina	= "%5d %3.0lf%% %s\n";
 char 	*cmdprocline	= "%3.0lf%% <%.66s>\n";
 /***************************************************************************/
-char	*schprochdr	= "\n  PID  TGID  NTHRD POLICY NICE PRIO RTPRIO "
-			  "CURCPU STATUS  S  %s CMD  %4d/%-4d\n";
-char 	*schproclina	= "%5d %5d %4d/%c %-6s %4d %4d %6d %6d %-6s "
-			  " %c %3.0lf%% %.14s\n";
-char 	*schprocline	= "                                             "
-			  "%s  E %3.0lf%% <%.12s>\n";
+char	*schprochdr	= "\n  PID  TGID  THR  POLICY NICE PRIO RTPRIO "
+			  "CURCPU ST EXC   S  %s CMD  %4d/%-4d\n";
+char 	*schproclina	= "%5d %5d %4d  %-6s %4d %4d %6d %6d %-2s       "
+			  "%c %3.0lf%% %.14s\n";
+char 	*schprocline	= "                                            "
+			  "%c%c %3ld   E %3.0lf%% <%.12s>\n";
+/***************************************************************************/
+char	*totuserhdr	= "\nNPROCS  SYSCPU  USRCPU  VSIZE  RSIZE  RDDSK "
+			  "WRDSK RNET SNET  %s USER %4d/%-4d\n";
+char 	*totuserlin	= "%s %s %s %s %s  %s %s %s %s "
+	                  "%3.0lf%% %.14s\n";
+/***************************************************************************/
+char	*totprochdr	= "\nNPROCS  SYSCPU  USRCPU  VSIZE  RSIZE  RDDSK "
+			  "WRDSK RNET SNET  %s CMD  %4d/%-4d\n";
+char 	*totproclin	= "%s %s %s %s %s  %s %s %s %s "
+	                  "%3.0lf%% %.14s\n";
 /***************************************************************************/
 
 /*
@@ -207,6 +235,7 @@ totalcap(struct syscap *psc, struct sstat *sstat, struct pstat *pstat, int nact)
 	register int	i;
 	count_t	pgsz = getpagesz();
 
+	psc->nrcpu      = sstat->cpu.nrcpu;
 	psc->availcpu	= sstat->cpu.all.stime +
 			  sstat->cpu.all.utime +
 			  sstat->cpu.all.ntime +
@@ -214,7 +243,6 @@ totalcap(struct syscap *psc, struct sstat *sstat, struct pstat *pstat, int nact)
 			  sstat->cpu.all.wtime +
 			  sstat->cpu.all.Itime +
 			  sstat->cpu.all.Stime;
-	psc->availcpu  /= sstat->cpu.nrcpu;
 
 	psc->availmem	= sstat->mem.physmem * pgsz/1024;
 
@@ -271,7 +299,7 @@ void
 showgenproc(struct pstat *curstat, double perc)
 {
 	struct passwd 	*pwd;
-	char		*username;
+	char		*username, exittype;
 
 	char		format1[16], format2[16];
 	char		format3[16], format4[16];
@@ -303,8 +331,8 @@ showgenproc(struct pstat *curstat, double perc)
 								  KBFORMAT),
 			   username,
 			   curstat->gen.nthr,
-			   curstat->gen.pid == curstat->gen.tgid ? 'L' : 'M',
-			   curstat->gen.excode & ~(LONG_MAX) ? "N--" : "---",
+			   curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
+			   '-',
 			   curstat->gen.state,
 			   perc, curstat->gen.name);
 		}
@@ -347,19 +375,27 @@ showgenproc(struct pstat *curstat, double perc)
 			else
 				username = pwd->pw_name;
 
-			sprintf(format3, "%c%c%c%3ld",
-				curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
-		  		curstat->gen.excode & 0xff        ? 'S' : 'E',
-		  		curstat->gen.excode & 0x80        ? 'C' : '-',
-		  		curstat->gen.excode & 0xff        ? 
-					 curstat->gen.excode     & 0x7f :
-			  		(curstat->gen.excode>>8) & 0xff );
+			if (curstat->gen.excode & 0xff)
+			{
+				if (curstat->gen.excode & 0x80)
+					exittype = 'C';
+				else
+					exittype = 'S';
+			}
+			else
+			{
+				exittype = 'E';
+			}
 
 			printg(genprocline1,
 			  val2cpustr(curstat->cpu.stime*1000/HZ, format1),
 			  val2cpustr(curstat->cpu.utime*1000/HZ, format2),
 			  username,
-			  format3,
+			  curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
+			  exittype,
+		  	  curstat->gen.excode & 0xff        ? 
+				 curstat->gen.excode     & 0x7f :
+			  	(curstat->gen.excode>>8) & 0xff,
 			  perc, curstat->gen.name);
 		}
 		else
@@ -561,44 +597,119 @@ shownetproc(struct pstat *curstat, double perc)
 		/*
 		** show process-info of exited process
 		*/
-	   if (kernpatch & PATCHACCT)
-	   {
-		printg(netproclinep,
-		  curstat->gen.pid,
-		  val2valstr(curstat->net.tcprcv,      format1,  6),
-		  val2valstr(avgtcpr,                  format2,  4),
-		  val2valstr(curstat->net.tcpsnd,      format3,  6),
-		  val2valstr(avgtcps,                  format4,  4),
-		  val2valstr(curstat->net.udprcv,      format5,  6),
-		  val2valstr(avgudpr,                  format6,  4),
-		  val2valstr(curstat->net.udpsnd,      format7,  6),
-		  val2valstr(avgudps,                  format8,  4),
-		  curstat->net.rawrcv,
-		  curstat->net.rawsnd,
-		  perc, curstat->gen.name);
-	   }
-	   else
-	   {
-		if (curstat->gen.pid == 0)
-			printg("    ? ");
-		else
-			printg("%5d ", curstat->gen.pid);
+	   	if (kernpatch & PATCHACCT)
+	   	{
+			printg(netproclinep,
+		  	curstat->gen.pid,
+		  	val2valstr(curstat->net.tcprcv,      format1,  6),
+		  	val2valstr(avgtcpr,                  format2,  4),
+		  	val2valstr(curstat->net.tcpsnd,      format3,  6),
+		  	val2valstr(avgtcps,                  format4,  4),
+		  	val2valstr(curstat->net.udprcv,      format5,  6),
+		  	val2valstr(avgudpr,                  format6,  4),
+		  	val2valstr(curstat->net.udpsnd,      format7,  6),
+		  	val2valstr(avgudps,                  format8,  4),
+		  	curstat->net.rawrcv,
+		  	curstat->net.rawsnd,
+		  	perc, curstat->gen.name);
+	   	}
+	   	else
+	   	{
+			if (curstat->gen.pid == 0)
+				printg("    ? ");
+			else
+				printg("%5d ", curstat->gen.pid);
 
-		printg(netprocline, perc, curstat->gen.name);
-	   }
+			printg(netprocline, perc, curstat->gen.name);
+	  	}
 	}
 }
 
+/*
+** print a line of total resource consumption per user
+*/
+void
+showtotuser(struct pstat *curstat, double perc)
+{
+	struct passwd 	*pwd;
+	char		numuser[16], *username;
+
+	char		format1[16], format2[16];
+	char		format3[16], format4[16];
+	char		format5[16], format6[16];
+	char		format7[16], format8[16];
+	char		format9[16];
+
+	/*
+	** determine the user-name of process
+	*/
+	if ( (pwd = getpwuid(curstat->gen.ruid)) == NULL)
+	{
+		snprintf(numuser, sizeof numuser, "%u", curstat->gen.ruid);
+		username = numuser;
+	}
+	else
+	{
+		username = pwd->pw_name;
+	}
+
+	printg(totuserlin,
+		val2valstr(curstat->gen.pid,           format1, 6),
+		val2cpustr(curstat->cpu.stime*1000/HZ, format2),
+		val2cpustr(curstat->cpu.utime*1000/HZ, format3),
+		val2memstr(curstat->mem.vmem *1024,    format4, KBFORMAT),
+		val2memstr(curstat->mem.rmem *1024,    format5, KBFORMAT),
+		val2valstr(curstat->dsk.rio,           format6, 5),
+		val2valstr(curstat->dsk.wio,           format7, 5),
+	   	val2valstr(curstat->net.tcprcv +
+		           curstat->net.udprcv +
+			   curstat->net.rawrcv,        format8, 4),
+		val2valstr(curstat->net.tcpsnd +
+			   curstat->net.udpsnd +
+			   curstat->net.rawsnd,        format9, 4),
+		perc, username);
+}
+
+/*
+** print a line of total resource consumption per program (process name)
+*/
+void
+showtotproc(struct pstat *curstat, double perc)
+{
+	char		format1[16], format2[16];
+	char		format3[16], format4[16];
+	char		format5[16], format6[16];
+	char		format7[16], format8[16];
+	char		format9[16];
+
+	printg(totproclin,
+		val2valstr(curstat->gen.pid,           format1, 6),
+		val2cpustr(curstat->cpu.stime*1000/HZ, format2),
+		val2cpustr(curstat->cpu.utime*1000/HZ, format3),
+		val2memstr(curstat->mem.vmem *1024,    format4, KBFORMAT),
+		val2memstr(curstat->mem.rmem *1024,    format5, KBFORMAT),
+		val2valstr(curstat->dsk.rio,           format6, 5),
+		val2valstr(curstat->dsk.wio,           format7, 5),
+	   	val2valstr(curstat->net.tcprcv +
+		           curstat->net.udprcv +
+			   curstat->net.rawrcv,        format8, 4),
+		val2valstr(curstat->net.tcpsnd +
+			   curstat->net.udpsnd +
+			   curstat->net.rawsnd,        format9, 4),
+		perc, curstat->gen.name);
+}
+
 /*
 ** print a line of (more or less) static info about one process
 */
 void
 showvarproc(struct pstat *curstat, double perc)
 {
-	char		format1[16], format2[16], format3[16];
+	char		format1[16], format2[16];
 	struct passwd 	*pwd;
 	struct group 	*grp;
 	char		*username, *groupname;
+	char		exittype;
 
 	/*
 	** determine the user- and group-name of the process
@@ -627,7 +738,8 @@ showvarproc(struct pstat *curstat, double perc)
 		  	groupname,
 			convdate(curstat->gen.btime, format1),
 			convtime(curstat->gen.btime, format2),
-			curstat->gen.excode & ~(LONG_MAX) ? "N--" : "---",
+			curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
+			'-',
 		  	curstat->gen.state,
 		  	perc, curstat->gen.name);
 	}
@@ -641,20 +753,28 @@ showvarproc(struct pstat *curstat, double perc)
 		else
 			printg("%5d ", curstat->gen.pid);
 
-		sprintf(format3, "%c%c%c%3ld",
-			curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
-		  	curstat->gen.excode & 0xff        ? 'S' : 'E',
-		  	curstat->gen.excode & 0x80        ? 'C' : '-',
-		  	curstat->gen.excode & 0xff        ? 
-				 curstat->gen.excode     & 0x7f :
-		  		(curstat->gen.excode>>8) & 0xff );
+		if (curstat->gen.excode & 0xff)
+		{
+			if (curstat->gen.excode & 0x80)
+				exittype = 'C';
+			else
+				exittype = 'S';
+		}
+		else
+		{
+			exittype = 'E';
+		}
 
 		printg(varprocline,
 		  	username,
 		  	groupname,
 			convdate(curstat->gen.btime, format1),
 			convtime(curstat->gen.btime, format2),
-		  	format3,
+			curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
+			exittype,
+			curstat->gen.excode & 0xff        ?
+				curstat->gen.excode     & 0x7f :
+				(curstat->gen.excode>>8) & 0xff,
 		  	perc, curstat->gen.name);
 	}
 }
@@ -706,8 +826,7 @@ showcmdproc(struct pstat *curstat, double perc)
 void
 showschproc(struct pstat *curstat, double perc)
 {
-	char	format1[16];
-	char	*pol;
+	char	*pol, exittype;
 
 	switch (curstat->cpu.policy)
 	{
@@ -736,13 +855,12 @@ showschproc(struct pstat *curstat, double perc)
 		  	curstat->gen.pid,
 		  	curstat->gen.tgid,
 		  	curstat->gen.nthr,
-			curstat->gen.pid == curstat->gen.tgid ? 'L' : 'M',
 		  	pol,
 		  	curstat->cpu.nice,
 		  	curstat->cpu.prio,
 		  	curstat->cpu.rtprio,
 		  	curstat->cpu.curcpu,
-			curstat->gen.excode & ~(LONG_MAX) ? "N--" : "---",
+			curstat->gen.excode & ~(LONG_MAX) ? "N-" : "--",
 		  	curstat->gen.state,
 		  	perc, curstat->gen.name);
 	}
@@ -756,16 +874,25 @@ showschproc(struct pstat *curstat, double perc)
 		else
 			printg("%5d", curstat->gen.pid);
 
-		sprintf(format1, "%c%c%c%3ld",
-			curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
-		  	curstat->gen.excode & 0xff        ? 'S' : 'E',
-		  	curstat->gen.excode & 0x80        ? 'C' : '-',
-		  	curstat->gen.excode & 0xff        ? 
-				 curstat->gen.excode     & 0x7f :
-		  		(curstat->gen.excode>>8) & 0xff );
+		if (curstat->gen.excode & 0xff)
+		{
+			if (curstat->gen.excode & 0x80)
+				exittype = 'C';
+			else
+				exittype = 'S';
+		}
+		else
+		{
+			exittype = 'E';
+		}
 
 		printg(schprocline,
-			format1, perc, curstat->gen.name);
+			curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
+			exittype,
+		 	curstat->gen.excode & 0xff        ? 
+				 curstat->gen.excode     & 0x7f :
+				(curstat->gen.excode>>8) & 0xff,
+			perc, curstat->gen.name);
 	}
 }
 
@@ -777,36 +904,44 @@ priphead(int curlist, int totlist, char *sortorder, char showtype)
 {
 	switch (showtype)
 	{
-	   case 'g':
+	   case MPROCGEN:
 		if ( !(kernpatch & PATCHSTAT) )
 			printg(genprochdr1, sortorder, curlist, totlist);
 		else
 			printg(genprochdr2, sortorder, curlist, totlist);
 		break;
 
-	   case 'm':
+	   case MPROCMEM:
 		printg(memprochdr, sortorder, curlist, totlist);
 		break;
 
-	   case 'd':
+	   case MPROCDSK:
 		printg(dskprochdr, sortorder, curlist, totlist);
 		break;
 
-	   case 'n':
+	   case MPROCNET:
 		printg(netprochdr, sortorder, curlist, totlist);
 		break;
 
-	   case 'v':
+	   case MPROCVAR:
 		printg(varprochdr, sortorder, curlist, totlist);
 		break;
 
-	   case 'c':
+	   case MPROCARG:
 		printg(cmdprochdr, sortorder, curlist, totlist);
 		break;
 
-	   case 's':
+	   case MPROCSCH:
 		printg(schprochdr, sortorder, curlist, totlist);
 		break;
+
+	   case MCUMUSER:
+		printg(totuserhdr, sortorder, curlist, totlist);
+		break;
+
+	   case MCUMPROC:
+		printg(totprochdr, sortorder, curlist, totlist);
+		break;
 	}
 }
 
@@ -815,7 +950,7 @@ priphead(int curlist, int totlist, char *sortorder, char showtype)
 ** print the list of processes from the deviation-list
 */
 int
-priproc(struct pstat  *pstat, int firstproc, int lastproc, int curline,
+priproc(struct pstat *pstat, int firstproc, int lastproc, int curline,
 	int curlist, int totlist, char *sortorder,
 	struct syscap *sb, char showtype,
 	struct selection *sel, int screen)
@@ -840,34 +975,60 @@ priproc(struct pstat  *pstat, int firstproc, int lastproc, int curline,
 		*/
 		switch (*sortorder) 
 		{
-		   case 'C':
+		   case MSORTCPU:
 			if (sb->availcpu)
+			{
 				perc = (double)(curstat->cpu.stime +
 				                curstat->cpu.utime  ) *
-					        100.0 / sb->availcpu;
+						100.0 /
+						(sb->availcpu / sb->nrcpu);
+
+				if (perc > 100.0 * sb->nrcpu)
+					perc = 100.0 * sb->nrcpu;
+
+				if (perc > 100.0 * curstat->gen.nthr)
+					perc = 100.0 * curstat->gen.nthr;
+			}
 			else
+			{
 				perc = 0.0;
+			}
 			break;
 
-		   case 'M':
+		   case MSORTMEM:
 			if (sb->availmem)
+			{
 				perc = (double)curstat->mem.rmem *
                                                100.0 / sb->availmem;
+
+				if (perc > 100.0)
+					perc = 100.0;
+			}
 			else
+			{
 				perc = 0.0;
+			}
 			break;
 
-		   case 'D':
+		   case MSORTDSK:
 			if (sb->availdsk)
+			{
 				perc = (double)(curstat->dsk.rio +
 				                curstat->dsk.wio  ) *
 			        		100.0 / sb->availdsk;
+
+				if (perc > 100.0)
+					perc = 100.0;
+			}
 			else
+			{
 				perc = 0.0;
+			}
 			break;
 
-		   case 'N':
+		   case MSORTNET:
 			if (sb->availnet)
+			{
 				perc = (double)(curstat->net.tcpsnd +
 						curstat->net.tcprcv +
 						curstat->net.udpsnd +
@@ -875,17 +1036,20 @@ priproc(struct pstat  *pstat, int firstproc, int lastproc, int curline,
 						curstat->net.rawsnd +
 						curstat->net.rawrcv ) *
 						100.0 / sb->availnet;
+
+				if (perc > 100.0)
+					perc = 100.0;
+			}
 			else
+			{
 				perc = 0.0;
+			}
 			break;
 
 		   default:
 			perc = 0.0;
 		}
 
-		if (perc > 100.0)
-			perc = 100.0;
-
 		/*
 		** check if only processes of a particular user
 		** should be shown
@@ -918,33 +1082,41 @@ priproc(struct pstat  *pstat, int firstproc, int lastproc, int curline,
 		*/
 		switch (showtype)
 		{
-		   case 'g':
+		   case MPROCGEN:
 			showgenproc(curstat, perc);
 			break;
 
-		   case 'm':
+		   case MPROCMEM:
 			showmemproc(curstat, perc);
 			break;
 
-		   case 'd':
+		   case MPROCDSK:
 			showdskproc(curstat, perc);
 			break;
 
-		   case 'n':
+		   case MPROCNET:
 			shownetproc(curstat, perc);
 			break;
 
-		   case 'v':
+		   case MPROCVAR:
 			showvarproc(curstat, perc);
 			break;
 
-		   case 'c':
+		   case MPROCARG:
 			showcmdproc(curstat, perc);
 			break;
 
-		   case 's':
+		   case MPROCSCH:
 			showschproc(curstat, perc);
 			break;
+
+		   case MCUMUSER:
+			showtotuser(curstat, perc);
+			break;
+
+		   case MCUMPROC:
+			showtotproc(curstat, perc);
+			break;
 		}
 
 		curline++;
@@ -980,28 +1152,15 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int fixedhead,
 
 	percputot = cputot / sstat->cpu.nrcpu;
 
-	if (osrel == 2 && osvers >= 6)
-	{
-		printg("CPU | sys %6.0f%% | user %6.0f%% | "
-		       "irq  %6.0f%% | idle %6.0f%% | wait %6.0f%% |\n",
-			(double) (sstat->cpu.all.stime * 100.0) / percputot,
-			(double)((sstat->cpu.all.utime + sstat->cpu.all.ntime)
-			                               * 100.0) / percputot,
-			(double)((sstat->cpu.all.Itime + sstat->cpu.all.Stime)
-			                               * 100.0) / percputot,
-			(double) (sstat->cpu.all.itime * 100.0) / percputot,
-			(double) (sstat->cpu.all.wtime * 100.0) / percputot);
-	}
-	else
-	{
-		printg("CPU | sys %6.0f%% | user %6.0f%% | "
-		       "nice %6.0f%% | idle %6.0f%% | cs %s |\n",
-			(double)(sstat->cpu.all.stime * 100.0) / percputot,
-			(double)(sstat->cpu.all.utime * 100.0) / percputot,
-			(double)(sstat->cpu.all.ntime * 100.0) / percputot,
-			(double)(sstat->cpu.all.itime * 100.0) / percputot,
-			val2valstr(sstat->cpu.csw, format1, 9));
-	}
+	printg("CPU | sys %6.0f%% | user %6.0f%% | "
+	       "irq  %6.0f%% | idle %6.0f%% | wait %6.0f%% |\n",
+		(double) (sstat->cpu.all.stime * 100.0) / percputot,
+		(double)((sstat->cpu.all.utime + sstat->cpu.all.ntime)
+		                               * 100.0) / percputot,
+		(double)((sstat->cpu.all.Itime + sstat->cpu.all.Stime)
+		                               * 100.0) / percputot,
+		(double) (sstat->cpu.all.itime * 100.0) / percputot,
+		(double) (sstat->cpu.all.wtime * 100.0) / percputot);
 
 	curline++;
 
@@ -1025,40 +1184,23 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int fixedhead,
 			if (percputot == 0)
 				percputot = 1; /* avoid divide-by-zero */
 
-			if (osrel == 2 && osvers >= 6)
-			{
-				printg("cpu | sys %6.0f%% | "
-				       "user %6.0f%% | irq  %6.0f%% | "
-				       "idle %6.0f%% | cpu%03d w%3.0f%% |\n",
-					(double) (sstat->cpu.cpu[i].stime 
-							* 100.0) / percputot,
-					(double)((sstat->cpu.cpu[i].utime +
-					          sstat->cpu.cpu[i].ntime)
-			                              	* 100.0) / percputot,
-					(double)((sstat->cpu.cpu[i].Itime +
-					          sstat->cpu.cpu[i].Stime)
-			                              	* 100.0) / percputot,
-					(double) (sstat->cpu.cpu[i].itime
-							* 100.0) / percputot,
-					sstat->cpu.cpu[i].cpunr,
-					(double) (sstat->cpu.cpu[i].wtime
-							* 100.0) / percputot);
-			}
-			else
-			{
-				printg("cpu | sys %6.0f%% | "
-				       "user %6.0f%% | nice %6.0f%% | "
-				       "idle %6.0f%% |       cpu%03d |\n",
-			   		(double)(sstat->cpu.cpu[i].stime
-							*100.0) /percputot,
-			   		(double)(sstat->cpu.cpu[i].utime
-							*100.0) /percputot,
-			   		(double)(sstat->cpu.cpu[i].ntime
-							*100.0) /percputot,
-			   		(double)(sstat->cpu.cpu[i].itime
-							*100.0) /percputot,
-					sstat->cpu.cpu[i].cpunr);
-			}
+			printg("cpu | sys %6.0f%% | "
+			       "user %6.0f%% | irq  %6.0f%% | "
+			       "idle %6.0f%% | cpu%03d w%3.0f%% |\n",
+				(double) (sstat->cpu.cpu[i].stime 
+						* 100.0) / percputot,
+				(double)((sstat->cpu.cpu[i].utime +
+				          sstat->cpu.cpu[i].ntime)
+			                       	* 100.0) / percputot,
+				(double)((sstat->cpu.cpu[i].Itime +
+				          sstat->cpu.cpu[i].Stime)
+			                       	* 100.0) / percputot,
+				(double) (sstat->cpu.cpu[i].itime
+						* 100.0) / percputot,
+				sstat->cpu.cpu[i].cpunr,
+				(double) (sstat->cpu.cpu[i].wtime
+						* 100.0) / percputot);
+
 			curline++;
 			lin++;
 		}
@@ -1262,3 +1404,23 @@ intfcompar(const void *a, const void *b)
 	if (apack > bpack) return -1;
 	                   return  0;
 }
+
+int
+compusr(const void *a, const void *b)
+{
+	register int uida = ((struct pstat *)a)->gen.ruid;
+	register int uidb = ((struct pstat *)b)->gen.ruid;
+
+	if (uida > uidb) return  1;
+	if (uida < uidb) return -1;
+	                 return  0;
+}
+
+int
+compnam(const void *a, const void *b)
+{
+	register char *nama = ((struct pstat *)a)->gen.name;
+	register char *namb = ((struct pstat *)b)->gen.name;
+
+	return strcmp(nama, namb);
+}
